// complexType[@name] sequence element[@name, @type] -- HttpHeaders

//-----------------
// Members
// attribute[@name, @type]    complexType element[@name]
// element[@name, @type]      sequence    complexType        element[@name]
// element[@name, @type]      sequence    complexType[@name] 'KeyValuePair (typical enum)
// element[@name]             complexType sequence           complexType     element[@name]
// element[@ref]              sequence    complexType        element[@name]
// enumeration[@value] restriction simpleType[@name] 'SortDirection
// element[@name]             sequence    complexType        element[@name] 'HTTPHeaders

package com.restphone

import scala.xml._
import XmlMethods._
import scala.annotation._

case class ReturnValue(types: List[WsdlType], enumerants: List[Enumerant], members: List[Member])

object TypeBuilder {
  val Element = new com.restphone.LabelMatcher("element")
  val ComplexType = new com.restphone.LabelMatcher("complexType")
  val Restriction = new LabelMatcher("restriction")
  val Name = new AttrMatcher("name")

  def toList(n: NodeSeq, parent: Option[WsdlType] = None): List[WsdlType] = {

    def getDescendents(e: Elem, parent: Option[WsdlType]) = e.child.flatMap { c => innerToList(c, parent) } toList

    def extractType(name: String, parent: Option[WsdlType], e: Elem) = {
      val wsdlType = WsdlType(name = name, parent = parent)
      // Handle types that are members of other types
      val ms = List(Member(name, wsdlType, ArgNumber(e)))
      val r = ReturnValue(types = List(wsdlType),
        enumerants = List(),
        members = ms)
      r :: getDescendents(e, Some(wsdlType))
    }

    def innerToList(n: NodeSeq, parent: Option[WsdlType] = None): List[ReturnValue] = {
      n match {
        case Element(e @ Name(n)) =>
          extractType(n, parent, e)
        case ComplexType(e @ Name(n)) =>
          extractType(n, parent, e)
        case e: Elem =>
          getDescendents(e, parent)
        case _ => List()
      }
    }

    innerToList(n).flatMap { _.types }
  }

  val x = <element name="bar"><element name="foo"/></element>

  def doit = toList(Parser.wsdl)
}
