package com.restphone

import scala.xml._
import StringMethods._
import ExceptionHelpers._
import util.control.Exception._

object ExceptionHelpers {
  val catchNFE = catching(classOf[NumberFormatException])
}

abstract class AbstractSurrounder {
  val surroundWith: (String, String)
  def render(s: String) = s match {
    case "" => ""
    case _ => surroundWith._1 + s + surroundWith._2
  }
  def render(s: Option[String]): String = s.map { render(_) } getOrElse ""
}
class Surrounder(val surroundWith: (String, String)) extends AbstractSurrounder
case object PassthroughSurrounder extends Surrounder(("", ""))
case object ParensSurrounder extends Surrounder(("(", ")"))
case object CurlyBraceSurrounder extends Surrounder(("{", "}"))
case object BracketSurrounder extends Surrounder(("[", "]"))

object Samples {
}

trait HasToDecl {
  def toDecl: String
}

object StringMethods {
  val ColonSplitter = "(.*?):(.*)".r
  def splitOnColon(s: String) = {
    s match {
      case ColonSplitter(a, b) => (Some(a), Some(b))
      case _ => None
    }
  }
  val InitialLetter = "(^.)(.*)".r
  val StartsWithHIT = "^HIT(.*)".r
  def toVariableName(s: String) = s match {
    case StartsWithHIT(hit, rest) => "hit" + rest
    case "Type" => "hitType"
    case InitialLetter(a, rest) => a.toLowerCase + rest
    case s => s.toLowerCase
  }

  val lparen = "("
  val rparen = ")"
  val lcurly = "{"
  val rcurly = "}"
  val pipe = "|"
  val comma = ","
  val equalsWithSpaces = " = "
  val colon = ":"
}

object Parser {
  import scala.xml._
  import XmlMethods._
  import StringMethods._

  import scala.annotation._

  val wsdl = scala.xml.XML.load(scala.xml.Source.fromFile("AWSMechanicalTurkRequester.wsdl.wsdl"))
}
