package com.restphone

import scala.xml._
import StringMethods._
import ExceptionHelpers._
import util.control.Exception._


sealed abstract class ArgNumber
case object ExactlyOne extends ArgNumber
case object IsOption extends ArgNumber
case object IsList extends ArgNumber

abstract sealed class MemberType
case class WsdlType(name: String, parent: Option[WsdlType] = None, members: List[Member] = List(), enumerants: List[Enumerant] = List()) extends MemberType
case class Enumerant(name: String)
case class Member(name: String, memberType: MemberType, number: ArgNumber)
case object ArgNumber {
  import XmlMethods._

  def apply(n: NodeSeq): ArgNumber = {
    val min = getAttr(n, "@minOccurs")
    val max = getAttr(n, "@maxOccurs")
    (min, max) match {
      case (_, Some("unbounded")) => IsList
      case (Some("0"), _) => IsOption
      case (_, _) => ExactlyOne
    }
  }
}


