package com.restphone

import scala.xml._
import StringMethods._
import ExceptionHelpers._
import util.control.Exception._

object XmlMethods {
  def hasElements(n: NodeSeq) = n.length > 0
  def getAttr(n: NodeSeq, attrName: String) = optionalString(n \ attrName text)
  def getAttrAsInt(n: NodeSeq, attrName: String) = optionalString(n \ attrName text) match {
    case Some(s) => allCatch opt s.toInt
    case None => None
  }
  def getName(n: NodeSeq) = getAttr(n, "@name")
  def getValueAttribute(n: NodeSeq) = getAttr(n, "@value")

  def optionalString(s: String) = if (s.equals("")) None else Some(s)
  def optionalString(n: NodeSeq): Option[String] = if (n.length > 0) optionalString(n.text) else None
  def optionalBigDecimal(n: NodeSeq) = optionalString(n) map { BigDecimal(_) }
  def optionalDouble(n: NodeSeq) = optionalString(n) map { _.toDouble }
  def optionalDateTime(n: NodeSeq) = optionalString(n) map { new org.joda.time.DateTime(_) }
  def optionalInt(n: NodeSeq) = optionalString(n) map { _.toInt }
  def optionalLong(n: NodeSeq) = optionalString(n) map { _.toLong }
  def optionalBoolean(n: NodeSeq) = optionalString(n) map { _.toBoolean }
}

abstract class XmlMatcher
class LabelMatcher(s: String) extends XmlMatcher {
  def unapply(n: NodeSeq): Option[Elem] = {
    n match {
      case e: Elem if e.label.equals(s) => Some(e)
      case _ => None
    }
  }
}
class AttrMatcher(attrNameWithoutAtSign: String) extends XmlMatcher {
  private val attrWithAtSign = "@" + attrNameWithoutAtSign
  def unapply(n: NodeSeq): Option[String] = {
    val a = n \ attrWithAtSign
    if (a.length > 0) Some(a.text) else None
  }
}

